from datetime import datetime
from openerp.osv import fields, osv
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT
import calendar
import logging

_logger = logging.getLogger(__name__)

MONTHS = zip(range(1, 13), calendar.month_name[1:])


def monthrange(year=None, month=None):
    today = datetime.today()
    y = year or today.year
    m = month or today.month
    return y, m, calendar.monthrange(y, m)[1]


class hr_duty_roster_shift(osv.Model):
    _name = 'hr.duty_roster_shift'

    def _get_days_default(self, cr, uid, context=None):
        if 'active_id' in context:
            active_id = context['active_id']
            row = self.pool.get('hr.duty_roster').read(cr, uid, active_id, [
                'year', 'month'])
            mrange = calendar.monthrange(row['year'], row['month'])
            return mrange[1]
        return 0

    def _get_days(self, cr, uid, ids, field_name, args=None, context=None):
        res = {}
        if ids:
            parent_row = self.read(cr, uid, ids[0], ['duty_roster_id'],
                                   context=context)
            row = self.pool.get('hr.duty_roster').read(
                cr,
                uid,
                parent_row['duty_roster_id'][0],
                ['year', 'month'],
                context=context)
            mrange = calendar.monthrange(row['year'], row['month'])
            for _id in ids:
                res[_id] = mrange[1]
        return res

    def _get_shift_codes(self, cr, uid, context=None):
        context = context or {}
        if 'shift_codes' not in context:
            model = self.pool.get('hr.shift_code')
            ids = model.search(cr, uid, [], order="code", context=context)
            ret = model.read(cr, uid, ids, ['code'], context=context)
            context['shift_codes'] = [(r['code'], r['code']) for r in ret]
        return context['shift_codes']

    _columns = {
        'duty_roster_id': fields.many2one(
            'hr.duty_roster', 'Duty Roster', required=True,
            ondelete="cascade"),
        'employee_id': fields.many2one(
            'hr.employee', 'Employee', required=True),
        'days': fields.function(
            _get_days, type="integer", string='Days', required=True),
        'day_1': fields.selection(_get_shift_codes, string='Day 1', size=1),
        'day_2': fields.selection(_get_shift_codes, string='Day 2', size=1),
        'day_3': fields.selection(_get_shift_codes, string='Day 3', size=1),
        'day_4': fields.selection(_get_shift_codes, string='Day 4', size=1),
        'day_5': fields.selection(_get_shift_codes, string='Day 5', size=1),
        'day_6': fields.selection(_get_shift_codes, string='Day 6', size=1),
        'day_7': fields.selection(_get_shift_codes, string='Day 7', size=1),
        'day_8': fields.selection(_get_shift_codes, string='Day 8', size=1),
        'day_9': fields.selection(_get_shift_codes, string='Day 9', size=1),
        'day_10': fields.selection(_get_shift_codes, string='Day 10', size=1),
        'day_11': fields.selection(_get_shift_codes, string='Day 1', size=1),
        'day_12': fields.selection(_get_shift_codes, string='Day 12', size=1),
        'day_13': fields.selection(_get_shift_codes, string='Day 13', size=1),
        'day_14': fields.selection(_get_shift_codes, string='Day 14', size=1),
        'day_15': fields.selection(_get_shift_codes, string='Day 15', size=1),
        'day_16': fields.selection(_get_shift_codes, string='Day 16', size=1),
        'day_17': fields.selection(_get_shift_codes, string='Day 17', size=1),
        'day_18': fields.selection(_get_shift_codes, string='Day 18', size=1),
        'day_19': fields.selection(_get_shift_codes, string='Day 19', size=1),
        'day_20': fields.selection(_get_shift_codes, string='Day 20', size=1),
        'day_21': fields.selection(_get_shift_codes, string='Day 21', size=1),
        'day_22': fields.selection(_get_shift_codes, string='Day 22', size=1),
        'day_23': fields.selection(_get_shift_codes, string='Day 23', size=1),
        'day_24': fields.selection(_get_shift_codes, string='Day 24', size=1),
        'day_25': fields.selection(_get_shift_codes, string='Day 25', size=1),
        'day_26': fields.selection(_get_shift_codes, string='Day 26', size=1),
        'day_27': fields.selection(_get_shift_codes, string='Day 27', size=1),
        'day_28': fields.selection(_get_shift_codes, string='Day 28', size=1),
        'day_29': fields.selection(_get_shift_codes, string='Day 29', size=1),
        'day_30': fields.selection(_get_shift_codes, string='Day 30', size=1),
        'day_31': fields.selection(_get_shift_codes, string='Day 31', size=1),
    }

    _defaults = {
        'days': _get_days_default,
    }

    _sql_constraints = [
        ('shift_uniq', 'unique(duty_roster_id, employee_id)',
         'Duplicate employee entries detected for this duty roster.')]


class hr_duty_roster(osv.Model):
    _name = 'hr.duty_roster'

    def _get_date(self, cr, uid, ids, name, args, context=None):
        results = {}
        for row in self.read(cr, uid, ids, ['year', 'month'], context=context):
            row_id = row['id']
            results[row_id] = datetime(
                year=row['year'], month=row['month'], day=1).isoformat()
        return results

    _columns = {
        'name': fields.char('Name', size=60, required=True),
        'month': fields.selection(MONTHS, 'Month', required=True),
        'year': fields.integer('Year', required=True),
        'date': fields.function(_get_date, type="datetime", method=True),
        'state': fields.selection(
            [('new', 'New'), ('draft', 'Draft'), ('pending', 'Pending'),
             ('checked', 'Checked')]),
        'shifts': fields.one2many(
            'hr.duty_roster_shift', 'duty_roster_id', string="Shifts"),
        'department_id': fields.many2one(
            'hr.department', 'Department', required=True),
        'checked_by_id': fields.many2one('hr.employee', 'Checked by'),
        'checked_dt': fields.datetime('Checked at'),
        'remarks': fields.text('Remarks')
    }

    _defaults = {
        'state': lambda self, cr, uid, context: 'new',
        'month': lambda self, cr, uid, context: monthrange()[1],
        'year': lambda self, cr, uid, context: monthrange()[0],
    }

    def has_shifts(self, cr, uid, ids, context=None):
        return self.pool.get('hr.duty_roster_shift').search(
            cr, uid, [('duty_roster_id', 'in', ids)], context=context,
            count=True)

    def is_checked(self, cr, uid, ids, context=None):
        return False

    def action_draft(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'draft'}, context)

    def action_pending(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'pending'}, context)

    def action_checked(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'checked'}, context)

    _sql_constraints = [
        ('uniq_duty_roster',
         'unique(department_id, name, year, month)',
         'Duty roster for the same department, month, year and name has already \
         been created.')]


class hr_shift_code(osv.Model):
    _rec_name = 'code'
    _name = 'hr.shift_code'

    def _time_diff(self, time_in, time_out):
        """Returns the number of minutes if both time in and time out
        are given."""

        duration = 0
        if time_out and time_in:
            time_in, time_out = [
                datetime.strptime(x, DEFAULT_SERVER_DATETIME_FORMAT) for x in (
                    time_in, time_out)]
            duration = (time_out - time_in).seconds / 60
        return int(duration)

    def _get_duration(self, cr, uid, ids, name, args, context=None):
        results = {}
        for row in self.read(cr, uid, ids, ['time_in', 'time_out'],
                             context=context):
            row_id = row['id']
            time_in, time_out = row['time_in'], row['time_out']
            results[row_id] = self._time_diff(time_in, time_out)
        return results

    def onchange_time(self, cr, uid, ids, time_in, time_out, context=None):
        ret = {}
        ret['value'] = {
            'duration': self._time_diff(time_in, time_out)
        }
        return ret

    _columns = {
        'code': fields.char('Code', size=1, required=True),
        'description': fields.char('Description', size=30),
        'time_in': fields.datetime('Time In'),
        'time_out': fields.datetime('Time Out'),
        'break': fields.char('Break', size=8),
        'duration': fields.function(_get_duration, string="Duration",
                                    type="integer", method=True),
    }

    _defaults = {
        'break': lambda self, cr, uid, context: '00:30:00',
    }
